using System;
using System.IO;
using Microsoft.Xna.Framework.Content.Pipeline;
// TODO: replace this with the type you want to import.
using OhsaiContentProcessors.Properties;
using TImport = OhsaiContentProcessors.DataContainer;

namespace OhsaiContentProcessors
{
    /// <summary>
    /// This class will be instantiated by the XNA Framework Content Pipeline
    /// to import a file from disk into the specified type, TImport.
    /// 
    /// This should be part of a Content Pipeline Extension Library project.
    /// 
    /// TODO: change the ContentImporter attribute to specify the correct file
    /// extension, display name, and default processor for this importer.
    /// </summary>
    [ContentImporter(".xml", DisplayName = "Ohsai Importer - Createdbyx.com", DefaultProcessor = "OhsaiProcessor")]
    public class OhsaiImporter : ContentImporter<TImport>
    {
        public override TImport Import(string filename, ContentImporterContext context)
        {
            // declare a varible for storing the data contained in the file
            string data = string.Empty;

            // report that asset processing has begun
            context.Logger.PushFile(filename);

            // try to read the file
            try
            {
                // read all file data as a string
                data = File.ReadAllText(filename);
            }
            catch (Exception ex)
            {
                // throw a InvalidContentException exception that reports the problem
                throw new InvalidContentException(Resources.CouldNotReadFile,
                                  new ContentIdentity(Path.GetFullPath(filename)), ex);
            }
            finally
            {
                // report that asset processing has completed
                context.Logger.PopFile();
            }

            // return the string data that was read as a DataContainer object.
            return new DataContainer(data);
        }
    }
}
