using System;
using cbxTileMapperReaders;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Ohsai;
using XNAGameCodeKit;

namespace Ohsai_TestProject.Components
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class Flocking : Microsoft.Xna.Framework.DrawableGameComponent, IExample
    {
        private Texture2D texture;
        private Ohsai.IAgent[] actors;
        SpriteBatch spriteBatch;
        private XNAGameCodeKit.LineDrawing drawHelper;
        private MapData mapData;


        public Flocking(Game game)
            : base(game)
        {
            this.drawHelper = this.GetService<XNAGameCodeKit.LineDrawing>();
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // TODO: Add your initialization code here

            base.Initialize();
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {

            base.Update(gameTime);
        }

        private void DrawActor(Ohsai.IAgent actor)
        {
            var pos = new Vector2(actor.Position.X, actor.Position.Y);
            this.drawHelper.DrawCircle(pos, 12, Color.Blue);
            pos = new Vector2(actor.Position.X, actor.Position.Y);
            var dir = pos + (new Vector2(actor.Direction.X, actor.Direction.Y) * 20);
            this.drawHelper.DrawCircle(dir, 5, Color.Red);
            this.drawHelper.DrawLine(pos, dir, Color.Red);
        }

        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            this.spriteBatch = new SpriteBatch(this.GraphicsDevice);
            this.texture = this.Game.Content.Load<Texture2D>("lompster");

            this.actors = new IAgent[10];
            for (int i = 0; i < this.actors.Length; i++)
            {
                var ai = this.Game.Content.Load<OhsaiContentReaders.ReaderResult>("FlockingBehavior", true);
                if (ai.Success)
                {
                    this.actors[i] = ai.Agent;

                }
            }
            this.mapData = this.Game.Content.Load<cbxTileMapperReaders.MapData>("TestMap");

            base.LoadContent();
        }

        /// <summary>
        /// Allows the game component to draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Draw(GameTime gameTime)
        {
            // draw actor
            foreach (var actor in this.actors)
            {
                this.DrawActor(actor);
            }
        }

        public string Title
        {
            get { return "Flocking"; }
        }

        public string Description
        {
            get { return "Demonstrates flocking behavior."; }
        }

        public new bool Enabled
        {
            get { return base.Enabled; }
            set
            {
                base.Enabled = value;
                var aiManager = this.GetService<Ohsai.Manager>();
                var rnd = new Random((int)DateTime.Now.Ticks);
                if(this.actors==null) return;
                for (int i = 0; i < this.actors.Length; i++)
                {
                    if (!value)
                    {
                        aiManager.Actors.Remove(this.actors[i]);
                    }
                    else
                    {
                        aiManager.Actors.Add(this.actors[i]);
                        var map = this.Game.Components.FindComponent<cbxTileMapper.TileMapComponent>();
                        mapData.SyncMapData(map);

                        this.actors[i].Direction = Vector3.Right;
                        //this.actors[i].Position = new Vector3(this.GraphicsDevice.Viewport.Width / 2f, this.GraphicsDevice.Viewport.Height / 2f, 0);
                        var vec = new Vector3(rnd.Next(100, this.GraphicsDevice.Viewport.Width - 100),
                                              rnd.Next(100, this.GraphicsDevice.Viewport.Height - 100),
                                              0);
                        this.actors[i].Position = vec;
                    }
                }
            }
        }
    }
}