using System;
using cbxTileMapperReaders;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Ohsai;
using XNAGameCodeKit;

namespace Ohsai_TestProject.Components
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class MoveTowardsExample : Microsoft.Xna.Framework.DrawableGameComponent, IExample
    {
        private Texture2D texture;
        private Ohsai.IAgent actor;
        SpriteBatch spriteBatch;
        private XNAGameCodeKit.LineDrawing drawHelper;
        private MapData mapData;


        public MoveTowardsExample(Game game)
            : base(game)
        {
            this.drawHelper = this.GetService<XNAGameCodeKit.LineDrawing>();
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // TODO: Add your initialization code here

            base.Initialize();
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            var input = this.GetService<CBXInput4XNA.IUserInputService>();
            var objs = this.GetService<WorldObjectManager>();
            if (objs.Objects.Count == 0) return;
            var target = objs.Objects[0];
            var stick = input.LeftThumbStick(PlayerIndex.One);
            target.Position += new Vector3(stick.X, -stick.Y, 0) * 0.5f * gameTime.ElapsedGameTime.Milliseconds;
        }

        private void DrawActor(Ohsai.IAgent actor)
        {
            var pos = new Vector2(actor.Position.X, actor.Position.Y);
            this.drawHelper.DrawCircle(pos, 12, Color.Blue);
            pos = new Vector2(actor.Position.X, actor.Position.Y);
            var dir = pos + (new Vector2(actor.Direction.X, actor.Direction.Y) * 20);
            this.drawHelper.DrawCircle(dir, 2, Color.Red);
            this.drawHelper.DrawLine(pos, dir, Color.Red);

            var act = actor as Actor;
            var rot = Math.Atan2(act.Direction.Y, act.Direction.X);
            var fov = MathHelper.ToRadians(act.FOV);
            var lPoint = new Vector2((float)Math.Cos(rot - (fov / 2)) * act.ViewDistance, (float)Math.Sin(rot - (fov / 2)) * act.ViewDistance);
            var rPoint = new Vector2((float)Math.Cos(rot + (fov / 2)) * act.ViewDistance, (float)Math.Sin(rot + (fov / 2)) * act.ViewDistance);
            this.drawHelper.DrawLine(pos, pos + lPoint, Color.White);
            this.drawHelper.DrawLine(pos, pos + rPoint, Color.White);

        }

        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            this.spriteBatch = new SpriteBatch(this.GraphicsDevice);
            this.texture = this.Game.Content.Load<Texture2D>("lompster");

            var ai = this.Game.Content.Load<OhsaiContentReaders.ReaderResult>("MoveTowardBehavior", true);
            if (ai.Success)
            {
                this.actor = ai.Agent;
                this.actor.WorldQuery = new WorldQuerying(this.Game);
            }

            this.mapData = this.Game.Content.Load<cbxTileMapperReaders.MapData>("TestMap");

            base.LoadContent();
        }

        /// <summary>
        /// Allows the game component to draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Draw(GameTime gameTime)
        {
            // draw actor
            this.DrawActor(this.actor);
            this.DrawObjects();
        }

        private void DrawObjects()
        {
            var objs = this.GetService<WorldObjectManager>();
            foreach (var baseObject in objs.Objects)
            {
                var actor = baseObject  ;
                var pos = new Vector2(actor.Position.X, actor.Position.Y);
                this.drawHelper.DrawCircle(pos, actor.CollisionRadius, Color.Green);
            }
        }

        public string Title
        {
            get { return "Move Towards"; }
        }

        public string Description
        {
            get { return "Demonstrates moving towards a target."; }
        }

        public new bool Enabled
        {
            get { return base.Enabled; }
            set
            {
                base.Enabled = value;
                var aiManager = this.GetService<Ohsai.Manager>();
                var objs = this.GetService<WorldObjectManager>();
                objs.Objects.Clear();
                var rnd = new Random((int)DateTime.Now.Ticks);
                if (!value)
                {
                    aiManager.Actors.Remove(this.actor);
                    var map = this.Game.Components.FindComponent<cbxTileMapper.TileMapComponent>();
                    map.Layers.Clear();
                }
                else
                {
                    aiManager.Actors.Add(this.actor);
                    var map = this.Game.Components.FindComponent<cbxTileMapper.TileMapComponent>();
                    this.mapData.SyncMapData(map);

                    this.actor.Direction = Vector3.Right;
                    this.actor.Position = new Vector3(this.GraphicsDevice.Viewport.Width / 2f, this.GraphicsDevice.Viewport.Height / 2f, 0);

                    var target = new BaseObject();
                    target.Position = this.actor.Position + (Vector3.Up * 25);
                    target.CollisionRadius = 5;
                    objs.Objects.Add(target);
                }
            }
        }
    }
}