using System;
using cbxTileMapperReaders;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using XNAGameCodeKit;

namespace Ohsai_TestProject.Components
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class ProximityExample : Microsoft.Xna.Framework.DrawableGameComponent, IExample
    {
        private Texture2D texture;
        private Actor actor;
        SpriteBatch spriteBatch;
        private XNAGameCodeKit.LineDrawing drawHelper;
        private MapData mapData;
        private Actor[] targets;


        public ProximityExample(Game game)
            : base(game)
        {
            this.drawHelper = this.GetService<XNAGameCodeKit.LineDrawing>();
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // TODO: Add your initialization code here

            base.Initialize();
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            var input = this.GetService<CBXInput4XNA.IUserInputService>();
            var objs = this.GetService<WorldObjectManager>();
            if (objs.Objects.Count == 0) return;
            var target = objs.Objects[0];
            var stick = input.LeftThumbStick(PlayerIndex.One);
            //target.Position += new Vector3(stick.X, -stick.Y, 0) * 0.5f * gameTime.ElapsedGameTime.Milliseconds;
            this.actor.Position += new Vector3(stick.X, -stick.Y, 0) * 0.5f * gameTime.ElapsedGameTime.Milliseconds;
        }

        private void DrawActor(Ohsai.IAgent actor, Color color)
        {
            var pos = new Vector2(actor.Position.X, actor.Position.Y);
            this.drawHelper.DrawCircle(pos, 12, color);
            pos = new Vector2(actor.Position.X, actor.Position.Y);
            var dir = pos + (new Vector2(actor.Direction.X, actor.Direction.Y) * 20);
            this.drawHelper.DrawCircle(dir, 2, Color.Red);
            this.drawHelper.DrawLine(pos, dir, Color.Red);

            var act = actor as Actor;
            var rot = Math.Atan2(act.Direction.Y, act.Direction.X);
            var fov = MathHelper.ToRadians(act.FOV);
            var lPoint = new Vector2((float)Math.Cos(rot - (fov / 2)) * act.ViewDistance, (float)Math.Sin(rot - (fov / 2)) * act.ViewDistance);
            var rPoint = new Vector2((float)Math.Cos(rot + (fov / 2)) * act.ViewDistance, (float)Math.Sin(rot + (fov / 2)) * act.ViewDistance);
            this.drawHelper.DrawLine(pos, pos + lPoint, Color.White);
            this.drawHelper.DrawLine(pos, pos + rPoint, Color.White);

            this.drawHelper.DrawCircle(pos, act.CollisionRadius, Color.Yellow);
        }

        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            this.spriteBatch = new SpriteBatch(this.GraphicsDevice);
            this.texture = this.Game.Content.Load<Texture2D>("lompster");

            var ai = this.Game.Content.Load<OhsaiContentReaders.ReaderResult>("ProximityMoveTowardBehavior", true);
            var worldQuery = new WorldQuerying(this.Game);
            if (ai.Success)
            {
                this.actor = ai.Agent as Actor;
                this.actor.WorldQuery = worldQuery;
            }

            this.targets = new Actor[1];
            for (int i = 0; i < targets.Length; i++)
            {
                var tmp = this.Game.Content.Load<OhsaiContentReaders.ReaderResult>("MoveToRandomLocations", true);
                if (tmp.Success)
                {
                    targets[i] = tmp.Agent as Actor;
                    targets[i].WorldQuery = worldQuery;
                }
            }

            this.mapData = this.Game.Content.Load<cbxTileMapperReaders.MapData>("TestMap");

            base.LoadContent();
        }

        /// <summary>
        /// Allows the game component to draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Draw(GameTime gameTime)
        {
            // draw actor
            this.DrawActor(this.actor, Color.Blue);
            foreach (var target in targets)
            {
                this.DrawActor(target, Color.Purple);
            }
        }

        public string Title
        {
            get { return "Proximity"; }
        }

        public string Description
        {
            get { return "Demonstrates moving towards any target\r\nthat is in proximity."; }
        }

        public new bool Enabled
        {
            get { return base.Enabled; }
            set
            {
                base.Enabled = value;
                var aiManager = this.GetService<Ohsai.Manager>();
                var objs = this.GetService<WorldObjectManager>();
                objs.Objects.Clear();
                var rnd = new Random((int)DateTime.Now.Ticks);
                if (!value)
                {
                    aiManager.Actors.Remove(this.actor);
                    objs.Objects.Remove(this.actor);
                    if (targets != null)
                    {
                        foreach (var target in this.targets)
                        {
                            aiManager.Actors.Remove(target);
                            objs.Objects.Remove(target);
                        }
                    }

                    var map = this.Game.Components.FindComponent<cbxTileMapper.TileMapComponent>();
                    map.Layers.Clear();
                }
                else
                {
                    aiManager.Actors.Add(this.actor);
                    objs.Objects.Add(this.actor);

                    foreach (var target in this.targets)
                    {
                        target.Position = new Vector3(rnd.Next(50, this.GraphicsDevice.Viewport.Width - 50),
                                                      rnd.Next(50, this.GraphicsDevice.Viewport.Height - 50), 0);
                        var dir = new Vector3(rnd.Next(0, this.GraphicsDevice.Viewport.Width),
                                              rnd.Next(0, this.GraphicsDevice.Viewport.Height), 0);
                        dir.Normalize();
                        target.Direction = dir;
                        aiManager.Actors.Add(target);
                        objs.Objects.Add(target);
                    }
                    var map = this.Game.Components.FindComponent<cbxTileMapper.TileMapComponent>();
                    this.mapData.SyncMapData(map);

                    this.actor.Direction = Vector3.Right;
                    this.actor.Position = new Vector3(this.GraphicsDevice.Viewport.Width / 2f, this.GraphicsDevice.Viewport.Height / 2f, 0);
                }
            }
        }
    }
}