using System;
using cbxTileMapperReaders;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Ohsai_TestProject.Components
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class RandomMovement : Microsoft.Xna.Framework.DrawableGameComponent, IExample
    {
        private Ohsai.IAgent actor;
        private XNAGameCodeKit.LineDrawing drawHelper;
        private MapData mapData;


        public RandomMovement(Game game)
            : base(game)
        {
            this.drawHelper = this.GetService<XNAGameCodeKit.LineDrawing>();
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // TODO: Add your initialization code here

            base.Initialize();
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {

            base.Update(gameTime);
        }

        private void DrawActor()
        {
            var pos = new Vector2(this.actor.Position.X, this.actor.Position.Y);
            this.drawHelper.DrawCircle(pos, 12, Color.Blue);
            pos = new Vector2(this.actor.Position.X, this.actor.Position.Y);
            var dir = pos + (new Vector2(this.actor.Direction.X, this.actor.Direction.Y) * 20);
            this.drawHelper.DrawCircle(dir, 5, Color.Red);
            this.drawHelper.DrawLine(pos, dir, Color.Red);
        }

        protected override void LoadContent()
        {
            var ai = this.Game.Content.Load<OhsaiContentReaders.ReaderResult>("GamePadMoveToRandomLocations");
            if (ai.Success)
            {
                this.actor = ai.Agent ;
                this.actor.WorldQuery = new WorldQuerying(this.Game);
            }

            this.mapData = this.Game.Content.Load<cbxTileMapperReaders.MapData>("TestMap");
                                   
            base.LoadContent();
        }

        /// <summary>
        /// Allows the game component to draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Draw(GameTime gameTime)
        {
            // draw actor
            this.DrawActor();
        }

        public string Title
        {
            get { return "Random Movement"; }
        }

        public string Description
        {
            get { return "Moves the actor to random\r\nlocations on the screen"; }
        }

        public new bool Enabled
        {
            get { return base.Enabled; }
            set
            {
                base.Enabled = value;
                var aiManager = this.GetService<Ohsai.Manager>();
                if (!value)
                {
                    aiManager.Actors.Remove(this.actor);
                    var map = this.Game.Components.FindComponent<cbxTileMapper.TileMapComponent>();
                    map.Layers.Clear();
                }
                else
                {
                    aiManager.Actors.Add(this.actor);
                    this.actor.Direction = Vector3.Right;
                    this.actor.Position = new Vector3(this.GraphicsDevice.Viewport.Width / 2f, this.GraphicsDevice.Viewport.Height / 2f, 0);
                    var map = this.Game.Components.FindComponent<cbxTileMapper.TileMapComponent>();
                    mapData.SyncMapData(map);
                }
            }
        }
    }
}