using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;


namespace Ohsai_TestProject.Components
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class SimpleUserInput : Microsoft.Xna.Framework.DrawableGameComponent ,IExample
    {
        private Texture2D texture;
        private  Ohsai.IAgent actor;
        SpriteBatch spriteBatch;


        public  new bool Enabled
        {
            get { return base.Enabled; }
            set
            {
                base.Enabled = value;
                var aiManager = this.GetService<Ohsai.Manager>();
                if (!value)
                {
                    aiManager.Actors.Remove(this.actor);
                }
                else
                {
                    aiManager.Actors.Add(this.actor);
                    this.actor.Direction = Vector3.Right;
                    this.actor.Position = Vector3.Zero;
                }
            }
        }

        public SimpleUserInput(Game game)
            : base(game)
        {
           }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // TODO: Add your initialization code here

            base.Initialize();
        }

        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);
            this.texture = this.Game.Content.Load<Texture2D>("lompster");

            var ai = this.Game.Content.Load<OhsaiContentReaders.ReaderResult>("SimpleUserInput");
            if (ai.Success)
            {
                this.actor = ai.Agent ;
            }

            base.LoadContent();
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            // TODO: Add your update code here

            base.Update(gameTime);
        }

        /// <summary>
        /// Allows the game component to draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Draw(GameTime gameTime)
        {
            this.spriteBatch.Begin();
            this.texture.Draw(this.spriteBatch, this.actor.Position.X, this.actor.Position.Y);
            this.spriteBatch.End();

            base.Draw(gameTime);
        }

        public string Title
        {
            get { return "Simple User Input"; }
        }

        public string Description
        {
            get { return "Allows the user to control the actor\r\nusing WSAD or gamepad left stick"; }
        }
    }
}