using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using Ohsai;
using XNAGameCodeKit;


namespace Ohsai_TestProject.Components
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class TimerExample : Microsoft.Xna.Framework.DrawableGameComponent, IExample
    {
        private Ohsai.IAgent actor;
        private XNAGameCodeKit.LineDrawing drawHelper;


        public TimerExample(Game game)
            : base(game)
        {
            this.drawHelper = this.GetService<XNAGameCodeKit.LineDrawing>();
        }


        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // TODO: Add your initialization code here

            base.Initialize();
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {

            base.Update(gameTime);
        }

        private void DrawActor()
        {
            var pos = new Vector2(this.actor.Position.X, this.actor.Position.Y);
            this.drawHelper.DrawCircle(pos, 12, Color.Blue);
            pos = new Vector2(this.actor.Position.X, this.actor.Position.Y);
            var dir = pos + (new Vector2(this.actor.Direction.X, this.actor.Direction.Y) * 20);
            this.drawHelper.DrawCircle(dir, 5, Color.Red);
            this.drawHelper.DrawLine(pos, dir, Color.Red);
        }

        protected override void LoadContent()
        {
            var ai = this.Game.Content.Load<OhsaiContentReaders.ReaderResult>("MoveAndRotateRight");
            if (ai.Success)
            {
                this.actor = ai.Agent;
            }
        }

        /// <summary>
        /// Allows the game component to draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Draw(GameTime gameTime)
        {
            // draw actor
            this.DrawActor();
        }

        public string Title
        {
            get { return "Timer Example"; }
        }

        public string Description
        {
            get { return "Moves and rotates the actor\r\nusing a timer"; }
        }

        public new bool Enabled
        {
            get { return base.Enabled; }
            set
            {
                base.Enabled = value;
                var aiManager = this.GetService<Ohsai.Manager>();
                if (!value)
                {
                    aiManager.Actors.Remove(this.actor);
                }
                else
                {
                    aiManager.Actors.Add(this.actor);
                    this.actor.Direction = Vector3.Right;
                    this.actor.Position = new Vector3(this.GraphicsDevice.Viewport.Width / 2f, this.GraphicsDevice.Viewport.Height / 2f, 0);
                }
            }
        }
    }
}