using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using XNAGameCodeKit;

namespace Ohsai_TestProject
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        private List<IExample> examples = new List<IExample>();
        private int currentExample = 0;
        private SpriteFont font;
        private bool runningExample = false;
        private XNAGameCodeKit.LineDrawing drawHelper;


        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            this.Content = new XNAGameCodeKit.ExtendedContentManager(this.Services);
            Content.RootDirectory = "Content";
            this.IsMouseVisible = true;
            graphics.PreferredBackBufferWidth = 1280;
            graphics.PreferredBackBufferHeight = 720;
            graphics.ApplyChanges();


            var input = new CBXInput4XNA.UserInputService(this);
            this.Components.Add(input);
            this.Services.AddService(typeof(CBXInput4XNA.IUserInputService), input);

            var bindings = new CBXInputBindings.ActionManagerComponent(this);
            this.Services.AddService(typeof(CBXInputBindings.ActionManager), bindings);
            bindings.Devices.Add(new CBXInputBindings.KeyboardDevice(input));
            bindings.Devices.Add(new CBXInputBindings.MouseDevice(input));
            bindings.Devices.Add(new CBXInputBindings.GamePadDevice(input));
            this.Components.Add(bindings);

            var objBindings = new CBXInputBindings.ObjectBindingManager(bindings);
            this.Services.AddService(objBindings.GetType(), objBindings);

            var map = new cbxTileMapper.TileMapComponent(this);
            this.Components.Add(map);

            var tileInfoManager = new cbxTileMapper.TileInfoManager(this);
            this.Components.Add(tileInfoManager);
            this.Services.AddService(tileInfoManager.GetType(), tileInfoManager);

            var aiManager = new Ohsai.Manager(this);
            this.Services.AddService(aiManager.GetType(), aiManager);
            this.Components.Add(aiManager);

            this.drawHelper = new LineDrawing(this);
            this.Components.Add(this.drawHelper);
            this.Services.AddService(this.drawHelper.GetType(), this.drawHelper);

            var objManager = new WorldObjectManager();
            this.Services.AddService(objManager.GetType(), objManager);

            // add example components
            this.examples.Add(new Components.SimpleUserInput(this) { Enabled = false, Visible = false });
            this.examples.Add(new Components.TimerExample(this) { Enabled = false, Visible = false });
            this.examples.Add(new Components.RandomMovement(this) { Enabled = false, Visible = false });
          //  this.examples.Add(new Components.Flocking(this) { Enabled = false, Visible = false });
            this.examples.Add(new Components.MoveTowardsExample(this) { Enabled = false, Visible = false });
            this.examples.Add(new Components.ProximityExample(this) { Enabled = false, Visible = false });

            foreach (IGameComponent example in this.examples)
            {
                this.Components.Add(example);
            }

        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here


            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);
            this.font = this.Content.Load<SpriteFont>("Kootenay");

            // load binding data
            var bindingData = this.Content.Load<BindingsContentReader.ReaderResult>("bindings");
            var am = this.Components.FindComponent<CBXInputBindings.ActionManagerComponent>();
            if (bindingData.Success) am.Bindings.Add(bindingData.Bindings);


            // load map data
            // var mapData = this.Content.Load<cbxTileMapperReaders.MapData>("TestMap");
            // var map = this.Components.FindComponent<cbxTileMapper.TileMapComponent>();
            // mapData.SyncMapData(map);
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            var input = this.Services.GetService<CBXInput4XNA.IUserInputService>();
            if (!this.runningExample)
            {

                if (input.KeyClick(Keys.Up))
                {
                    this.currentExample--;
                    if (this.currentExample < 0) this.currentExample = 0;
                }

                if (input.KeyClick(Keys.Down))
                {
                    this.currentExample++;
                    if (this.currentExample == this.examples.Count) this.currentExample = this.examples.Count - 1;
                }

                if (input.KeyClick(Keys.Enter))
                {
                    var com = this.examples[this.currentExample];
                    com.Visible = com.Enabled = true;
                    this.runningExample = true;
                }
            }

            if (input.KeyClick(Keys.Escape))
            {
                var com = this.examples[this.currentExample];
                com.Visible = com.Enabled = false;
                this.runningExample = false;
            }

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            if (!this.runningExample)
            {
                this.spriteBatch.Begin();
                string data = string.Empty;
                for (int i = 0; i < this.examples.Count; i++)
                {
                    data += (this.currentExample == i ? "*" : " ") + " " + (i + 1) + " - " + this.examples[i].Title + "\r\n";
                }
                var size = this.font.MeasureString(data);
                var pos = new Vector2((this.GraphicsDevice.Viewport.Width / 8f),
                                      (this.GraphicsDevice.Viewport.Height / 2f) - (size.Y / 2));
                this.spriteBatch.DrawString(this.font, data, pos, Color.White);


                data = this.examples[this.currentExample].Description;
                size = this.font.MeasureString(data);
                pos = new Vector2((this.GraphicsDevice.Viewport.Width / 2f),
                                  (this.GraphicsDevice.Viewport.Height / 2f) - (size.Y / 2));

                this.spriteBatch.DrawString(this.font, data, pos, Color.White);
                this.spriteBatch.End();
            }

            base.Draw(gameTime);
        }
    }
}
