﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using cbxTileMapper;
using Microsoft.Xna.Framework;

namespace Ohsai_TestProject
{
    public struct MapTileItem
    {
        Tile Tile { get; set; }
        string LayerName { get; set; }
        public Point TileIndex { get; set; }
        public Vector2 TilePosition { get; set; }

        public MapTileItem(Tile tile, int ix, int iy, float xpos, float ypos, string layerName)
            : this()
        {
            this.Tile = tile;
            this.TileIndex = new Point(ix, iy);
            this.TilePosition = new Vector2(xpos, ypos);
            this.LayerName = layerName;
        }
    }

    public class WorldQuerying : Ohsai.IWorldQuery
    {
        public Game Game { get; private set; }
        XNAGameCodeKit.TriangleScanlineCalculator tri = new XNAGameCodeKit.TriangleScanlineCalculator();

        public WorldQuerying(Game game)
        {
            this.Game = game;
        }

        public object[] GetObjects(Vector3 position, float distance, float fov, Vector3 direction)
        {
            var view = new BoundingFrustum(Matrix.Identity);
            view = view.BuildFrustum(this.Game.GraphicsDevice, Vector3.Up, fov, 0.1f, distance, position, position + direction);

            var map = this.Game.Components.FindComponent<cbxTileMapper.TileMapComponent>();
            if (map == null) return null;

            var objs = new object[1000];
            int index = 0;

            foreach (var mapLayerPair in map.Layers)
            {
                var tw = mapLayerPair.Value.TileWidth;
                var th = mapLayerPair.Value.TileHeight;
                var tileXCount = (int)((distance * 2) / tw);
                var tileYCount = (int)((distance * 2) / tw);
                var posX = (int)(position.X / tw);
                var posY = (int)(position.Y / th);
                posX -= tileXCount / 2;
                posY -= tileYCount / 2;

                for (int idx = posX; idx < posX + tileXCount; idx++)
                {
                    for (int idy = posY; idy < posY + tileYCount; idy++)
                    {
                        var tile = mapLayerPair.Value.GetTile(idx, idy);

                        if (tile != Tile.Empty)
                        {
                            var min = new Vector3((idx * tw) - (tw / 2), (idy * tw) - (th / 2), float.MinValue);
                            var max = new Vector3((idx * tw) + (tw / 2), (idy * tw) + (th / 2), float.MaxValue);
                            if (view.Intersects(new BoundingBox(min, max)))
                            {
                                if (index + 1 > objs.Length - 1)
                                {
                                    Array.Resize(ref objs, objs.Length + 1000);
                                }
                                objs[index++] = new MapTileItem(tile, idx, idy, idx * mapLayerPair.Value.TileWidth, idy * mapLayerPair.Value.TileHeight, mapLayerPair.Key);
                            }
                        }
                    }
                }
            }

            // add any collisions with world objects
            var objMan = this.Game.Services.GetService<WorldObjectManager>();
            foreach (var baseObject in objMan.Objects)
            {
                if (baseObject.Intersects(view))
                {
                    if (index + 1 > objs.Length - 1)
                    {
                        Array.Resize(ref objs, objs.Length + 1000);
                    }
                    objs[index++] = baseObject;
                }
            }


            Array.Resize(ref objs, index);
            return objs;
        }

        public object[] GetObjects(Vector3 position, float raduis)
        {
            var map = this.Game.Components.FindComponent<cbxTileMapper.TileMapComponent>();
            if (map == null) return null;
            var sphere = new BoundingSphere(position, raduis);

            var objs = new object[1000];
            int index = 0;
            foreach (var mapLayerPair in map.Layers)
            {
                var tw = mapLayerPair.Value.TileWidth;
                var th = mapLayerPair.Value.TileHeight;
                var tileXCount = (int)((raduis * 2) / tw);
                var tileYCount = (int)((raduis * 2) / tw);
                var posX = (int)(position.X / tw);
                var posY = (int)(position.Y / th);
                posX -= tileXCount / 2;
                posY -= tileYCount / 2;

                for (int idx = posX; idx < posX + tileXCount; idx++)
                {
                    for (int idy = posY; idy < posY + tileYCount; idy++)
                    {
                        var tile = mapLayerPair.Value.GetTile(idx, idy);
                        if (tile != Tile.Empty)
                        {
                            var min = new Vector3((idx * tw) - (tw / 2), (idy * tw) - (th / 2), float.MinValue);
                            var max = new Vector3((idx * tw) + (tw / 2), (idy * tw) + (th / 2), float.MaxValue);
                            if (sphere.Intersects(new BoundingBox(min, max)))
                            {
                                if (index + 1 > objs.Length - 1)
                                {
                                    Array.Resize(ref objs, objs.Length + 1000);
                                }
                                objs[index++] = new MapTileItem(tile, idx, idy, idx * mapLayerPair.Value.TileWidth, idy * mapLayerPair.Value.TileHeight, mapLayerPair.Key);
                            }
                        }
                    }
                }
            }

            // add any collisions with world objects
            var objMan = this.Game.Services.GetService<WorldObjectManager>();
            foreach (var baseObject in objMan.Objects)
            {
                if (baseObject.Intersects(sphere))
                {
                    if (index + 1 > objs.Length - 1)
                    {
                        Array.Resize(ref objs, objs.Length + 1000);
                    }
                    objs[index++] = baseObject;
                }
            }

            Array.Resize(ref objs, index);
            return objs;
        }
    }
}
