using EventArgsType = System.EventArgs;
using System.Collections.Generic;

namespace Ohsai
{
    [OhsaiAction(Name = "BaseAction", Description = "Provides a default action type.")]
    public class BaseAction : IAction
    {
        public virtual IVerb Verb { get; set; }
        public virtual List<IModifier> Modifiers { get; set; }


        public BaseAction()
        {
            this.Modifiers = new List<IModifier>();
        }

        public virtual void Update(object sender, EventArgsType e)
        {
            if (this.Verb != null)
            {
                this.Verb.Update(sender, e);

                foreach (var modifier in this.Modifiers)
                {
                    modifier.Update(sender, e);
                    var result = modifier.Apply(this.Verb);
                    if (result == ModifierResult.Halt) break;
                }
            }
        }
    }
}