using System;
using System.Collections.Generic;

namespace Ohsai
{
    public class BaseCondition : ICondition
    {
        public event EventHandler Passed;
        public event EventHandler Failed;

        public virtual ISensor Sensor { get; set; }
        public virtual List<ISensorFilter> Filters { get; set; }


        public BaseCondition()
        {
            this.Filters = new List<ISensorFilter>();
        }

        public void OnPassed()
        {
            if (this.Passed != null) this.Passed(this, EventArgs.Empty);
        }

        public void OnFailed()
        {
            if (this.Failed != null) this.Failed(this, EventArgs.Empty);
        }

        public virtual void Update(object sender, EventArgs e)
        {
            if (this.Sensor != null)
            {
                // update sensor
                this.Sensor.Update(sender, e);

                // filter sensor results
                foreach (var filter in this.Filters)
                    filter.Filter(this.Sensor);

                // check if there are any results left
                if (this.Sensor.Results.Count != 0)
                {
                    this.OnPassed();
                }
                else
                {
                    this.OnFailed();
                }
            }
        }
    }
}