using EventArgsType = System.EventArgs;

namespace Ohsai
{
    [OhsaiResponder(Name = "BaseResponder")]
    public class BaseResponder : IResponder
    {
        private ICondition condition;

        public BaseResponder()
        {
            this.Enabled = true;    
        }

        public virtual ICondition Condition
        {
            get { return this.condition; }
            set
            {
                if (this.condition != null)
                {
                    this.condition.Passed -= this.condition_Passed;
                    this.condition.Failed -= this.condition_Failed;
                }

                this.condition = value;

                if (this.condition != null)
                {
                    this.condition.Passed += this.condition_Passed;
                    this.condition.Failed += this.condition_Failed;
                }
            }
        }

        private void condition_Failed(object sender, EventArgsType e)
        {
            // do nothing if failed condition
        }

        private void condition_Passed(object sender, EventArgsType e)
        {
            if (this.Action != null) this.Action.Update(sender, e);
        }

        public virtual IAction Action { get; set; }

        public virtual void Update(object sender, EventArgsType e)
        {
            if(!this.Enabled) return;
            if (this.Condition != null)
            {
                this.Condition.Update(sender, e);
            }
            else
            {
                if (this.Action != null) this.Action.Update(this.condition, e);
            }
        }

        public bool Enabled { get; set; }

        public virtual void Reset()
        {
            if (this.condition != null && this.condition.Sensor != null && this.condition.Sensor.Results != null)
            {
                this.condition.Sensor.Results.Clear();
            }
        }
    }
}