using Ohsai.Sensors;

namespace Ohsai.SensorFilters
{
    [OhsaiSensorFilter("GreaterThan", "Provides a equality filter that filters results based on a value.",
        new[] { "Ohsai.IPosition", "Ohsai.Sensors.GamePadResult" })]
    public class GreaterThan : ISensorFilter
    {
        [OhsaiProperty("Value", "Gets or sets a value to compare against.")]
        public float Value { get; set; }
        [OhsaiAgentProperty("Owner", "Gets or sets the agent that owns this filter.")]
        public IAgent Owner { get; set; }


        public void Filter(ISensor sensor)
        {
            if (sensor == null) return;
            if (sensor.Results == null || sensor.Results.Count == 0) return;

            int index = 0;
            while (index < sensor.Results.Count)
            {
                var result = sensor.Results[index];
                if (result is Sensors.GamePadItem)
                {
                    index = this.FilterGamePadResult(sensor, index, result);
                }
                if (result is IPosition && this.Owner != null)
                {
                    var vector = ((IPosition)result).Position - this.Owner.Position;
                    if (vector.Length() < this.Value)
                    {
                        sensor.Results.RemoveAt(index);
                    }
                    else
                    {
                        index++;
                    }
                }
                else
                {
                    // unknown item so remove it?
                    sensor.Results.RemoveAt(index);
                }
            }
        }

        private int FilterGamePadResult(ISensor sensor, int index, object result)
        {
            var item = (Sensors.GamePadItem)result;
            switch (item.Source)
            {
                case GamePadSource.ButtonBig:
                case GamePadSource.ButtonA:
                case GamePadSource.ButtonB:
                case GamePadSource.ButtonX:
                case GamePadSource.ButtonY:
                case GamePadSource.LeftSholder:
                case GamePadSource.RightSholder:
                case GamePadSource.BackButton:
                case GamePadSource.StartButton:
                case GamePadSource.DPadUp:
                case GamePadSource.DPadDown:
                case GamePadSource.DPadLeft:
                case GamePadSource.DPadRight:
                case GamePadSource.LeftStickButton:
                case GamePadSource.RightStickButton:
                    // ignore buttons ?    
                    break;

                case GamePadSource.LeftTrigger:
                case GamePadSource.RightTrigger:
                case GamePadSource.LeftStickXPosition:
                case GamePadSource.RightStickXPosition:
                case GamePadSource.LeftStickYPosition:
                case GamePadSource.RightStickYPosition:
                    if (item.Value < this.Value)
                    {
                        sensor.Results.RemoveAt(index);
                    }
                    else
                    {
                        index++;
                    }
                    break;

                case GamePadSource.LeftStickPosition:
                case GamePadSource.RightStickPosition:
                    var length = item.Position.Length();
                    if (length < this.Value)
                    {
                        sensor.Results.RemoveAt(index);
                    }
                    else
                    {
                        index++;
                    }
                    break;
            }
            return index;
        }
    }
}