using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;

namespace Ohsai.Sensors
{
    public struct KeyboardItem
    {
        public Keys Source { get; set; }
        public PlayerIndex Player { get; set; }
        public float Value { get; set; }

        public KeyboardItem(Keys source, PlayerIndex player, float value)
            : this()
        {
            this.Source = source;
            this.Player = player;
            this.Value = value;
        }
    }

    [OhsaiSensor(Name = "KeyboardSensor", Description = "Provides a sensor for keyboard input.")]
    public class KeyboardSensor : BaseSensor
    {
        [OhsaiProperty(Name = "Player", Description = "Gets or sets the player to detect input from.")]
        public PlayerIndex Player { get; set; }
        [OhsaiProperty(Name = "Source", Description = "Gets or sets the keyboard input source.")]
        public Keys Source { get; set; }


        public KeyboardSensor()
        {
            this.Player = PlayerIndex.One;
        }

        public override void Update(object sender, EventArgs e)
        {
            this.Results.Clear();
            var kb = Keyboard.GetState(this.Player);
            if (kb.IsKeyDown(this.Source)) this.Results.Add(new KeyboardItem(this.Source, this.Player, 1));
        }
    }
}