using System;

namespace Ohsai.Sensors
{
    [Ohsai.OhsaiSensor( "TimerSensor",  "Provides a sensor for detecting a timer change.")]
    public class TimerSensor : Ohsai.BaseSensor
    {
        private TimeSpan lastTime;
        private bool firstRun = true;


        public TimerSensor()
        {
            
        }

        public TimerSensor(TimeSpan interval)
        {
            this.Interval = interval;
        }

        [Ohsai.OhsaiProperty("Interval",  "Determins how often the sensor will return a TimeSpan result.")]
        public virtual TimeSpan Interval { get; set; }

        public override void Reset()
        {
            this.firstRun = true;
            this.lastTime = TimeSpan.Zero;
        }

        public override void Update(object sender, EventArgs e)
        {
            if (this.firstRun)
            {
                this.lastTime = TimeSpan.FromTicks(DateTime.Now.Ticks);
                this.firstRun = false;
            }
            else
            {
                var value = TimeSpan.FromTicks(DateTime.Now.Ticks);
                if (value > this.lastTime + this.Interval)
                {
                    this.Results.Add(value);
                    this.lastTime = value;
                }
                else
                {
                    this.Results.Clear();
                }
            }
            base.Update(sender, e);
        }
    }
}