using System;
#if XNA
using VectorType = Microsoft.Xna.Framework.Vector3;
#endif
namespace Ohsai.VerbModifiers
{
    [OhsaiModifier("DirectionModifier", "Changes the direction on a agent that implements the IAgent interface.",
        new[] { "Ohsai.Verbs.Move" })]
    public class DirectionModifier : IModifier
    {
        [OhsaiReader("Ohsai.ReadersWriters.VectorTypeReader")]
        [OhsaiWriter("Ohsai.ReadersWriters.VectorTypeWriter")]
        [OhsaiProperty("Direction", "Gets or sets the direction.")]
        public virtual VectorType Direction { get; set; }
        [OhsaiProperty("InvertYAxis", "Gets or sets weatther or not the Y axis will be inverted.")]
        public virtual bool InvertYAxis { get; set; }



        public DirectionModifier()
        {

        }

        public virtual void Update(object sender, EventArgs e)
        {
            // ******************************************
            // TODO: This is horrible code and I need to find a more elegant solution for binding gamepad input to
            // TODO: the direction property. The reason is that sensor results can contain ANY type of data
            // TODO: thus this code makes an assumption that whatever sensor was used would have returned a
            // TODO: GamePadItem type in the results list.
            // ******************************************

            // check if there is a GamePadItem entry in the sensor results and if so
            // use the sum of the values
            var condition = sender as ICondition;
            if (condition == null) return;
            if (condition.Sensor != null)
            {
                var dir = new VectorType();
                bool foundItems = false;
                foreach (var result in condition.Sensor.Results)
                {
                    if (result is Sensors.GamePadItem)
                    {
                        var item = (Sensors.GamePadItem)result;
                        dir += new VectorType(item.Position.X, item.Position.Y , item.Position.Z);
                        foundItems = true;
                    }
                }
                if (foundItems) this.Direction = dir;
            }
            // ******************************************
        }

        public virtual ModifierResult Apply(IVerb verb)
        {
            if (verb is Verbs.Move)
            {
                var move = verb as Verbs.Move;
                move.Direction = this.InvertYAxis ? new VectorType(this.Direction.X, -this.Direction.Y, this.Direction.Z) : this.Direction;
            }

            return ModifierResult.Continue;
        }
    }
}