namespace Ohsai.VerbModifiers
{
    [Ohsai.OhsaiModifier(Name = "MoveAway", Description = "Moves a agent away from a target.",
        Types = new[] { "Ohsai.Verbs.Move" })]
    public class MoveAway : MoveModifier
    {
        [OhsaiAgentProperty("Owner", "Gets or sets the agent that this verb modifier will affect.")]
        public virtual IAgent Owner { get; set; }
       

        public MoveAway()
        {
            this.Distance = 1;
        }

        public override void Update(object sender, System.EventArgs e)
        {
            // ******************************************
            // TODO: This is horrible code and I need to find a more elegant solution for binding gamepad input to
            // TODO: the direction property. The reason is that sensor results can contain ANY type of data
            // TODO: thus this code makes an assumption that whatever sensor was used would have returned a
            // TODO: GamePadItem type in the results list.
            // ******************************************

            // check if there is a GamePadItem entry in the sensor results and if so
            // use the sum of the values
            var condition = sender as ICondition;
            if (condition == null) return;
            if (condition.Sensor != null)
            {
                foreach (var result in condition.Sensor.Results)
                {
                    if (result is IPosition && result != this.Owner)
                    {
                        // change direction of owner towards target
                        var pos = result as IPosition;
                        var dir = this.Owner.Position - pos.Position;
                        var length = this.Owner.Direction.Length();
                        dir.Normalize();
                        dir *= length;
                        this.Owner.Direction = dir;
                        break;
                    }
                }
            }
            // ******************************************
        }
    }
}