using System;

namespace Ohsai.VerbModifiers
{
    [OhsaiModifier("RotateModifier", "Provides a modifier type for rotating a agent.", new[] { "Ohsai.Verbs.Move" })]
    public class RotateModifier : IModifier
    {
        [OhsaiProperty("Rotation", "Gets or sets the rotation in degrees that will be applied.")]
        public virtual float Rotation { get; set; }
        [OhsaiProperty("AbsoluteRotation", "Gets or sets weather the rotation that is applied is a absolute or relative.")]
        public bool AbsoluteRotation { get; set; }

        public RotateModifier()
        {

        }

        public RotateModifier(float rotation)
        {
            this.Rotation = rotation;
        }

        public virtual void Update(object sender, EventArgs e)
        {
        }

        public virtual ModifierResult Apply(IVerb verb)
        {
            if (verb is Verbs.Move)
            {
                var move = verb as Verbs.Move;

                var dir = move.Direction;
                var distance = dir.Length();
                var currentDir = Math.Atan2(dir.Y, dir.X);
                var radian = this.Rotation * (Math.PI / 180f);
                dir.X = (float)(Math.Cos(this.AbsoluteRotation ? radian : currentDir + radian) * distance);
                dir.Y = (float)(Math.Sin(this.AbsoluteRotation ? radian : currentDir + radian) * distance);
                move.Direction = dir;
            }

            return ModifierResult.Continue;
        }
    }
}